package org.rcisoft.bus.storegoods.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.bus.storegoods.entity.StoreGoods;
import org.rcisoft.bus.storegoods.service.StoreGoodsService;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
* Created by cy on 2024年5月29日 下午1:21:49.
*/
@RestController
@RequestMapping("/storegoods")
public class StoreGoodsController extends CyPaginationController<StoreGoods> {


    @Autowired
    private StoreGoodsService storeGoodsServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:goods:add')")
    @CyOpeLogAnno(title = "system-商品信息表管理-新增商品信息表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加商品信息表", notes="添加商品信息表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid StoreGoods storeGoods, BindingResult bindingResult) {
        CyPersistModel data = storeGoodsServiceImpl.persist(storeGoods);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
                storeGoods);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:goods:delete')")
    @CyOpeLogAnno(title = "system-商品信息表管理-删除商品信息表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="逻辑删除商品信息表", notes="逻辑删除商品信息表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId, StoreGoods storeGoods) {
        storeGoods.setBusinessId(businessId);
        CyPersistModel data = storeGoodsServiceImpl.removeLogical(storeGoods);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:goods:delete')")
    @CyOpeLogAnno(title = "system-商品信息表管理-删除商品信息表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除商品信息表", notes="删除商品信息表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId, StoreGoods storeGoods) {
        storeGoods.setBusinessId(businessId);
        CyPersistModel data = storeGoodsServiceImpl.remove(storeGoods);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:goods:update')")
    @CyOpeLogAnno(title = "system-商品信息表管理-修改商品信息表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改商品信息表", notes="修改商品信息表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid StoreGoods storeGoods, BindingResult bindingResult) {
        storeGoods.setBusinessId(businessId);
        CyPersistModel data = storeGoodsServiceImpl.merge(storeGoods);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
                storeGoods);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:goods:query')")
    @CyOpeLogAnno(title = "system-商品信息表管理-查询商品信息表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一商品信息表", notes="查询单一商品信息表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        storeGoodsServiceImpl.findById(businessId));
    }
    @CyOpeLogAnno(title = "system-商品信息表管理-查询商品信息表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一商品信息表", notes="查询单一商品信息表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detailGoods/{businessId:\\w+}")
    public CyResult detailGoods(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                storeGoodsServiceImpl.findByOss(businessId));
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:goods:list')")
    @CyOpeLogAnno(title = "system-商品信息表管理-查询商品信息表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询商品信息表集合", notes="查询商品信息表集合")
    @GetMapping(value = "/listAll")
    public CyResult listAll(StoreGoods storeGoods) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            storeGoodsServiceImpl.findAll(storeGoods));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:goods:list')")
    @CyOpeLogAnno(title = "system-商品信息表管理-查询商品信息表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询商品信息表集合", notes="分页查询商品信息表集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(StoreGoods storeGoods) {
        storeGoodsServiceImpl.findAllByPagination(getPaginationUtility(), storeGoods);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-商品信息表管理-查询商品信息表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出商品信息表信息", notes = "导出商品信息表信息")
    @GetMapping(value = "/export")
    public void outWmsGoods(HttpServletResponse response, StoreGoods storeGoods, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="商品信息表信息.xls";break;
            case "1": excelName="商品信息表信息.xlsx";break;
            case "2": excelName="商品信息表信息.csv";break;
        }
        List<StoreGoods> storeGoodsList = storeGoodsServiceImpl.export(storeGoods);
        CyEpExcelUtil.exportExcel(storeGoodsList, "商品信息表信息", "商品信息表信息", StoreGoods.class, excelName, response);
    }

    @CyOpeLogAnno(title = "system-商品信息表管理-查询商品", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询商品", notes="查询商品")
    @GetMapping(value = "/detailByCode")
    public CyResult detailByCode(StoreGoods storeGoods) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                storeGoodsServiceImpl.queryWmsGoods(storeGoods));
    }
}
