package org.rcisoft.bus.storegoods.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.bus.storegoods.dao.StoreGoodsRepository;
import org.rcisoft.bus.storegoods.entity.StoreGoods;
import org.rcisoft.bus.storegoods.service.StoreGoodsService;
import org.rcisoft.bus.storegoodsapply.entity.StoreGoodsApply;
import org.rcisoft.bus.storegoodsstock.dao.StoreGoodsStockRepository;
import org.rcisoft.bus.storegoodsstock.entity.StoreGoodsStock;
import org.rcisoft.bus.wmsgoodsstock.entity.WmsGoodsStock;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.ossInfo.entity.OssInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by cy on 2024年5月29日 下午1:21:49.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class StoreGoodsServiceImpl extends ServiceImpl<StoreGoodsRepository, StoreGoods> implements StoreGoodsService {

    @Value("${system.goods_h5}")
    private String goodsH5;

    @Autowired
    private StoreGoodsStockRepository storeGoodsStockRepository;

    /**
     * 保存 商品信息表
     *
     * @param storeGoods
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(StoreGoods storeGoods) {
        // 批量插入
        String goodsCode = storeGoods.getGoodsCode();
        String[] goodStr = goodsCode.split(":");
        if(goodStr.length < 2) {
            throw new CyServiceException(1001, "数据不合法");
        }
        String[] itemCode = goodStr[1].split(",");
        // 循环
        for (int i = 0; i < itemCode.length; i++) {
            StoreGoods existingGoods = baseMapper.selectOne(new QueryWrapper<StoreGoods>().eq("goods_code",
                    goodStr[0] + itemCode[i]));
            if (existingGoods != null) {
                throw new CyServiceException(1001, "商品已存在");
            }
            StoreGoods entity = new StoreGoods();
            BeanUtil.copyProperties(storeGoods, entity);
            entity.setGoodsSize(itemCode[i]);
            entity.setGoodsCode(goodStr[0] + itemCode[i]);
            //增加操作
            int line = baseMapper.insert(entity);
            //插入图片
            if(StringUtils.isNotBlank(storeGoods.getUrlIdList())){
                List<Map<String, String>> params = new ArrayList<>();
                String[] oss_ids = storeGoods.getUrlIdList().split(",");
                Map<String, String> param = null;
                for (String id : oss_ids) {
                    param = new HashMap<>();
                    param.put("gid", entity.getBusinessId() + "");
                    param.put("oid" , id);
                    params.add(param);
                }
                baseMapper.insertOss(params);
            }
            log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                    storeGoods.getBusinessId() + "的商品信息表信息");
        }

        return new CyPersistModel(1);
    }

    /**
     * 删除 商品信息表
     *
     * @param storeGoods
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(StoreGoods storeGoods) {
        int line = baseMapper.realDelete(storeGoods);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                storeGoods.getBusinessId() + "的商品信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 商品信息表
     *
     * @param storeGoods
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(StoreGoods storeGoods) {
        storeGoods.setDeleted();
        int line = baseMapper.deleteById(storeGoods);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                storeGoods.getBusinessId() + "的商品信息表信息");
        return new CyPersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel delOss(StoreGoods storeGoods) {
        storeGoods.setDeleted();
        int line = baseMapper.delOssById(storeGoods);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                storeGoods.getBusinessId() + "的商品信息表信息");
        return new CyPersistModel(line);
    }
    /**
     * 修改 商品信息表
     *
     * @param storeGoods
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(StoreGoods storeGoods) {
        QueryWrapper<StoreGoods> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("goods_code", storeGoods.getGoodsCode());
        queryWrapper.ne("del_flag", "1");
        queryWrapper.ne("business_id", storeGoods.getBusinessId());
        List<StoreGoods> existingGoods = baseMapper.selectList(queryWrapper);
        if (existingGoods.size() > 0) {
            throw new CyServiceException(1001, "商品已存在");
        }
        int line = baseMapper.updateById(storeGoods);
        // 删除一下 goods_oss by goods
        baseMapper.delOssById(storeGoods);
        //插入图片
        if(StringUtils.isNotBlank(storeGoods.getUrlIdList())){
            List<Map<String, String>> params = new ArrayList<>();
            String[] oss_ids = storeGoods.getUrlIdList().split(",");
            Map<String, String> param = null;
            for (String id : oss_ids) {
                param = new HashMap<>();
                param.put("gid", storeGoods.getBusinessId() + "");
                param.put("oid" , id);
                params.add(param);
            }
            baseMapper.insertOss(params);
        }
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + storeGoods.getBusinessId() + "的商品信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 商品信息表
     *
     * @param id
     * @return
     */
//    @Override
//    public WmsGoods findById(int id) {
//        WmsGoods wmsGoods = baseMapper.selectById(id);
//        ossInfoMapper.selectBatchIds()
//        return wmsGoods;
//    }
    @Override
    public StoreGoods findById(int id) {
        return baseMapper.selectById(id);
    }

    /**
     * 分页查询 商品信息表
     *
     * @param storeGoods
     * @return
     */
    @Override
    public IPage<StoreGoods> findAllByPagination(CyPageInfo<StoreGoods> paginationUtility,
                                                 StoreGoods storeGoods) {
        IPage<StoreGoods> wmsGoodsIPage = baseMapper.queryWmsGoodssPaged(paginationUtility, storeGoods);
        wmsGoodsIPage.getRecords().forEach(record->{
            record.setOssInfoList(baseMapper.findOss(record));
        });
        return wmsGoodsIPage;
    }
    /**
     * 查询list 商品信息表
     *
     * @param storeGoods
     * @return
     */
    @Override
    public List<StoreGoods> findAll(StoreGoods storeGoods) {
        return baseMapper.queryWmsGoods(storeGoods);

    }

    /**
     * 导出商品信息表
     *
     * @return
     */
    @Override
    public List<StoreGoods> export(StoreGoods storeGoods) {
        List<StoreGoods> storeGoodsList = baseMapper.queryWmsGoods(storeGoods);
        if(CollectionUtils.isNotEmpty(storeGoodsList)){
            storeGoodsList.forEach(x->{
                x.setGoodsH5(goodsH5 + x.getBusinessId());
            });
        }
        return storeGoodsList;
    }

    @Override
    public List<StoreGoods> findByApplyId(StoreGoodsApply wmsGoodsLog) {

        return baseMapper.queryWmsGoodsByAppId(wmsGoodsLog);
    }

    @Override
    public List<StoreGoods> findByIdGoods(int id) {
        StoreGoods storeGoods = new StoreGoods();
        storeGoods.setGoodsId(id);
        return baseMapper.findByIdGoods(storeGoods);
    }

    @Override
    public StoreGoods findByOss(int id) {
        StoreGoods goods = baseMapper.selectById(id);
        if(goods != null) {
            goods.setOssInfoList(baseMapper.findOss(goods));
        }
        return goods;
    }

    @Override
    public List<OssInfo> findOss(StoreGoods storeGoods) {
        return baseMapper.findOss(storeGoods);
    }

    @Override
    public StoreGoods queryWmsGoods(StoreGoods storeGoods) {
        StoreGoods param = new StoreGoods();
        param.setActualGoodsCode(storeGoods.getGoodsCode());
        // 1. 查询 goodsNumber
        List<StoreGoods> result = baseMapper.queryWmsGoods(param);
        if(CollectionUtils.isEmpty(result) || result.size() > 1) {
            throw new CyServiceException(500, "商品不存在！");
        }
        StoreGoods goods = result.get(0);
        // 2. 查询 kwh
        StoreGoodsStock storeGoodsStock = storeGoodsStockRepository.selectByIdOut(goods.getBusinessId());
        return goods;
    }
}
