package org.rcisoft.bus.storegoodsstock.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.rcisoft.bus.storegoodslog.dao.StoreGoodsLogRepository;
import org.rcisoft.bus.storegoodslog.entity.StoreGoodsLog;
import org.rcisoft.bus.storegoodsstock.dao.StoreGoodsStockRepository;
import org.rcisoft.bus.storegoodsstock.dto.StoreGoodsStockPageResultsDto;
import org.rcisoft.bus.storegoodsstock.entity.StoreGoodsStock;
import org.rcisoft.bus.storegoodsstock.service.StoreGoodsStockService;
import org.rcisoft.bus.wmsgoods.entity.WmsGoods;
import org.rcisoft.bus.wmsgoodslog.dao.WmsGoodsLogRepository;
import org.rcisoft.bus.wmsgoodslog.entity.WmsGoodsLog;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Created by cy on 2024年5月29日 下午1:21:49.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class StoreGoodsStockServiceImpl extends ServiceImpl<StoreGoodsStockRepository, StoreGoodsStock>  implements StoreGoodsStockService {

    @Autowired
    StoreGoodsLogRepository wmsGoodsLogRepository;

    /**
    * 保存 库存管理表
    * @param storeGoodsStock
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(StoreGoodsStock storeGoodsStock){
        //增加操作
        int line = baseMapper.insert(storeGoodsStock);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            storeGoodsStock.getBusinessId()+"的库存管理表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 库存管理表
    * @param storeGoodsStock
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(StoreGoodsStock storeGoodsStock){
        int line = baseMapper.realDelete(storeGoodsStock);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            storeGoodsStock.getBusinessId()+"的库存管理表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 库存管理表
    * @param storeGoodsStock
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(StoreGoodsStock storeGoodsStock){
        storeGoodsStock.setDeleted();
        int line = baseMapper.deleteById(storeGoodsStock);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                storeGoodsStock.getBusinessId()+"的库存管理表信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 库存管理表
    * @param storeGoodsStock
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(StoreGoodsStock storeGoodsStock){
        StoreGoodsStock wmsOrigin = baseMapper.selectById(storeGoodsStock.getBusinessId());
        int line = baseMapper.updateById(storeGoodsStock);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ storeGoodsStock.getBusinessId()+"的库存管理表信息");
        StoreGoodsLog wmsGoodsLog = new StoreGoodsLog();
        wmsGoodsLog.setGoodsId(storeGoodsStock.getBusinessId());
        wmsGoodsLog.setGoodsCounts(wmsOrigin.getGoodsCounts());
        wmsGoodsLog.setGoodsName(wmsOrigin.getGoodsName());
        wmsGoodsLog.setGoodsCode(wmsOrigin.getGoodsCode());
        wmsGoodsLog.setIoType("3");
        wmsGoodsLog.setUpdateCounts(storeGoodsStock.getGoodsCounts());
        wmsGoodsLogRepository.insert(wmsGoodsLog);
        return new CyPersistModel(line);
    }
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel updateCounts(StoreGoodsStock storeGoodsStock) {
        int line = baseMapper.updateById(storeGoodsStock);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ storeGoodsStock.getBusinessId()+"的库存管理表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 库存管理表
    * @param id
    * @return
    */
    @Override
    public StoreGoodsStock findById(int id){
        return baseMapper.selectById(id);
    }
    @Override
    public StoreGoodsStock findByIdOut(int id){
        return baseMapper.selectByIdOut(id);
    }
    /**
    * 分页查询 库存管理表
    * @param storeGoodsStock
    * @return
    */
    @Override
    public IPage<StoreGoodsStockPageResultsDto> findAllByPagination(CyPageInfo<StoreGoodsStockPageResultsDto> paginationUtility,
                                                                    StoreGoodsStockPageResultsDto storeGoodsStock){
        return baseMapper.queryWmsGoodsStocksPaged(paginationUtility,storeGoodsStock);

    }

    /**
    * 查询list 库存管理表
    * @param storeGoodsStock
    * @return
    */
    @Override
    public List<StoreGoodsStock> findAll(StoreGoodsStock storeGoodsStock){
        return baseMapper.queryWmsGoodsStocks(storeGoodsStock);

    }

    /**
    * 导出库存管理表
    * @return
    */
    @Override
    public List<StoreGoodsStock> export(StoreGoodsStock storeGoodsStock) {
        List<StoreGoodsStock> storeGoodsStockList = baseMapper.queryWmsGoodsStocks(storeGoodsStock);
        return storeGoodsStockList;
    }

    @Override
    public StoreGoodsStock queryWmsGoodsStock(WmsGoods wmsGoods) {
        StoreGoodsStock stock = new StoreGoodsStock();
        stock.setActualGoodsCode(wmsGoods.getGoodsCode());
        stock.setActualGtGoodsCounts(0);
        // 1. 查询 goodsNumber
        List<StoreGoodsStock> result = baseMapper.queryWmsGoodsStocks(stock);
        if(CollectionUtils.isEmpty(result) || result.size() > 1 || result.get(0).getGoodsCounts() < 1) {
            throw new CyServiceException(500, "商品不存在或库存不足！");
        }
        return result.get(0);
    }

}
